/**
 * @file Motion.cpp
 * @author Jonathan Schulze, Nick Hübenthal, Hans Haupt
 * @brief Implementation of the Motion class.
 * @version 0.1
 * @date 2023-12-13
 * 
 * @copyright Copyright (c) 2023
 * 
 */

#include "Motion.h"

TaskHandle_t xMoveTaskHandle = NULL;
TaskHandle_t xClockwiseTaskHandle = NULL;
TaskHandle_t xAntiClockwiseTaskHandle = NULL;

// Initialize the movement component.
void Motion::begin(void) {
    
};
void Motion::moveTask(void * args) {
    analogWrite(MOTOR_LEFT_PIN, LEFT_MOTOR_DUTY);
    analogWrite(MOTOR_RIGHT_PIN, RIGHT_MOTOR_DUTY);
    vTaskDelay((uint32_t) args / portTICK_PERIOD_MS);
    analogWrite(MOTOR_LEFT_PIN, 0);
    analogWrite(MOTOR_RIGHT_PIN, 0);
    vTaskDelete(xMoveTaskHandle);
};

// Move forward for a certain amount of time.
void Motion::move(uint32_t moveForMs) {
    if (moveForMs > 0){
    xTaskCreate(moveTask, "Move", 4096, (void*)moveForMs, 10, &xMoveTaskHandle);
    } else{
        analogWrite(MOTOR_LEFT_PIN, LEFT_MOTOR_DUTY);
        analogWrite(MOTOR_RIGHT_PIN, RIGHT_MOTOR_DUTY);
    }
};

void Motion::leftMotorTask(void * args) {
    analogWrite(MOTOR_LEFT_PIN, LEFT_MOTOR_DUTY);
    analogWrite(MOTOR_RIGHT_PIN, 0);
    vTaskDelay((uint32_t) args / portTICK_PERIOD_MS);
    analogWrite(MOTOR_LEFT_PIN, 0);
    vTaskDelete(xClockwiseTaskHandle);
};

// Rotate clockwise for a certain amount of time.
void Motion::rotateClockwise(uint32_t rotateForMs) {
    if (rotateForMs > 0){
        xTaskCreate(leftMotorTask, "LeftMotor", 4096, (void*)rotateForMs, 10, &xClockwiseTaskHandle);
    } else {
        analogWrite(MOTOR_LEFT_PIN,LEFT_MOTOR_DUTY);
        analogWrite(MOTOR_RIGHT_PIN,0);
    }
};

void Motion::rightMotorTask(void * args) {
    analogWrite(MOTOR_RIGHT_PIN, RIGHT_MOTOR_DUTY);
    analogWrite(MOTOR_LEFT_PIN,0);
    vTaskDelay((uint32_t) args / portTICK_PERIOD_MS);
    analogWrite(MOTOR_RIGHT_PIN, 0);
    vTaskDelete(xAntiClockwiseTaskHandle);
};

// Rotate anticlockwise for a certain amount of time.
void Motion::rotateAntiClockwise(uint32_t rotateForMs) {
    if(rotateForMs > 0){
        xTaskCreate(rightMotorTask, "RightMotor", 4096, (void*)rotateForMs, 10, &xAntiClockwiseTaskHandle);
    } else {
        analogWrite(MOTOR_RIGHT_PIN,RIGHT_MOTOR_DUTY);
        analogWrite(MOTOR_LEFT_PIN,0);
    }
};

void Motion::stop(void){
    analogWrite(MOTOR_LEFT_PIN,0);
    analogWrite(MOTOR_RIGHT_PIN,0);
}
 
