/**
 * @file Motion.h
 * @author Jonathan Schulze, Nick Hübenthal
 * @brief This component controls the ability to rotate and change position. 
 * @version 0.1
 * @date 2023-12-13
 * 
 * @copyright Copyright (c) 2023
 * 
 */

#ifndef Motion_h
#define Motion_h
#include <stdint.h>
#include <Arduino.h>
#include <freertos/FreeRTOS.h>
#include <freertos/task.h>

class Motion{
protected:
    static const uint8_t RIGHT_MOTOR_DUTY = 128;
    static const uint8_t LEFT_MOTOR_DUTY = 128;
    static const int MOTOR_RIGHT_PIN = 11; 
    static const int MOTOR_LEFT_PIN = 12;
    static void moveTask(void * args);
    static void leftMotorTask(void * args);
    static void rightMotorTask(void * args);

public:
    /**
     * @brief Initialize the movement component.
     * 
    */
    void begin(void);

    /**
     * @brief Move forward for a certain amount of time.
     * Call with moveForMs 0 will start movement, that must be stopped explicit by call to stop().
     * @param moveForMs Representing the duration of forward moving in milliseconds.
    */
    static void move(uint32_t moveForMs=0);

    /**
     * @brief Rotate clockwise for a certain amount of time.
     * Call with moveForMs 0 will start movement, that must be stopped explicit by call to stop().
     * @param rotateForMs Representing the duration of rotating clockwise in milliseconds.
    */
    static void rotateClockwise(uint32_t rotateForMs=0);
    
    /**
     * @brief Rotate anticlockwise for a certain amount of time.
     * Call with moveForMs 0 will start movement, that must be stopped explicit by call to stop().
     * @param rotateForMs Representing the duration of rotating anticlockwise in milliseconds.
    */
    static void rotateAntiClockwise(uint32_t rotateForMs=0);

    /**
     * @brief stops any current movement, no matter if timebased or endless
     * 
     */
    static void stop(void);

};
#endif //Motion_h